﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FloydSteinberg
{
    public partial class FormMain : Form
    {
        Bitmap m_Source = null;
        Bitmap m_Result = null;

        public FormMain()
        {
            InitializeComponent();
        }

        private void buttonGenerate_Click(object sender, EventArgs e)
        {
            if (textBoxInput.Text != "")
            {
                using (Bitmap temp = Bitmap.FromFile(textBoxInput.Text) as Bitmap)
                {
                    m_Source = new Bitmap(temp.Width, temp.Height);

                    using (Graphics g = Graphics.FromImage(m_Source))
                    {
                        g.DrawImage(temp, 0, 0);
                        List<Color> palette = new List<Color>();
                        palette.Add(Color.FromArgb(0, 0, 0));
                        palette.Add(Color.FromArgb(255, 0, 0));
                        palette.Add(Color.FromArgb(0, 255, 0));
                        palette.Add(Color.FromArgb(0, 0, 255));
                        palette.Add(Color.FromArgb(255, 255, 0));
                        palette.Add(Color.FromArgb(255, 0, 255));
                        palette.Add(Color.FromArgb(0, 255, 255));
                        palette.Add(Color.FromArgb(255, 255, 255));
                        m_Result = FloydSteinbergDither.Process(m_Source, palette.ToArray());
                    }
                }
            }
            panelCanvas.Invalidate();
        }



        private void panelCanvas_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            Rectangle rect = new Rectangle(0, 0, 0, 0);

            if (m_Source != null)
            {
                rect.Size = m_Source.Size;
                g.DrawImage(m_Source, rect);
                rect.Y += m_Source.Height + 5;
            }

            if (m_Result != null)
            {
                rect.Size = m_Result.Size;
                g.DrawImage(m_Result, rect);
            }
        }


        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (m_Result == null)
            {
                MessageBox.Show("Please generate image first", this.Text);
                return;
            }

            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                string fileName = Path.GetFileNameWithoutExtension(saveFileDialog1.FileName);
                string filePath = saveFileDialog1.FileName;
                string savePath = fileName;

                savePath = filePath.Replace(fileName, fileName + "_source");
                m_Source.Save(savePath, ImageFormat.Png);

                savePath = filePath.Replace(fileName, fileName + "_result");
                m_Result.Save(savePath, ImageFormat.Png);
            }
        }
        

        private void textBoxInput_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                textBoxInput.Text = openFileDialog1.FileName;
            }
        }
    }
}
